from re import compile

split_template_tokens = compile(
    r"(\s{%-.*?-%}\s|\s{{-.*?-}}\s|\s{#-.*?-#}\s|\s{%-.*?%}|\s{{-.*?}}|\s{#-.*?#}|{%.*?-%}\s|{{.*?-}}\s|{#.*?-#}\s|{%.*?%}|{{.*?}}|{#.*?#})"
).split


var_name_checker = compile(r"[_a-zA-Z]\w*$")

is_message_start = compile(r"<\|\s?(user|system|assistant)\s?(\w{1,64})?\s?\|>")


def is_not_valid(name: str):
    return not var_name_checker.match(name)


def ensure_valid(name):
    if is_not_valid(name):
        raise NameError(name)



from typing import Any, AsyncIterator, Awaitable, Callable, Iterator

from promplate.prompt.chat import Message

CompleteText = Callable[[str, Any], str]
CompleteChat = Callable[[list[Message], Any], str]
GenerateText = Callable[[str, Any], Iterator[str]]
GenerateChat = Callable[[list[Message], Any], Iterator[str]]
Complete = CompleteText | CompleteChat
Generate = GenerateText | GenerateChat

AsyncCompleteText = Callable[[str, Any], Awaitable[str]]
AsyncCompleteChat = Callable[[list[Message], Any], Awaitable[str]]
AsyncGenerateText = Callable[[str, Any], AsyncIterator[str]]
AsyncGenerateChat = Callable[[list[Message], Any], AsyncIterator[str]]
AsyncComplete = AsyncCompleteText | AsyncCompleteChat
AsyncGenerate = AsyncGenerateText | AsyncGenerateChat
